<?php

namespace Drupal\schemaapp\Cache;

/**
 * Caching for a Schema App resource.
 */
class SchemaAppCache {

  const CACHE_ID = "schemaapp_cache:";
  const APIBASE = "https://data.schemaapp.com/";

  /**
   * User's account id.
   *
   * @var string
   */
  protected $accountId;
  /**
   * Users api key.
   *
   * @var string
   */
  protected $apiKey;

  /**
   * Page URL.
   *
   * @var string
   */
  protected $pageUrl;

  /**
   * Cache Lifetime.
   *
   * @var int
   */
  protected $cacheLifeTime;


  /**
   * Request Timeout.
   *
   * @var int
   */
  protected $requestTimeout;

  /**
   * Schema App Cache.
   *
   * Cache model for a particular page,
   * uses the URL and specified config for lookup.
   *
   * @param string $pageUrl
   *   The full URL for a page including query and fragment.
   * @param array $config
   *   Array with account_id, api_key, and use_url_params options.
   */
  public function __construct(string $pageUrl, array $config) {

    $this->pageUrl = $pageUrl;

    $this->accountId = $config['account_id'];
    $this->apiKey = $config['api_key'];
    $this->urlParams = $config['use_url_params'] ?? FALSE;

    $this->cacheLifeTime = $config['cache_lifetime'] ?? 86400;
    $this->cacheLifeTime = max(1, $this->cacheLifeTime);

    $this->requestTimeout = $config['api_timeout'] ?? 30;
    $this->requestTimeout = max(1, $this->requestTimeout);

    $urlComponents = parse_url($this->pageUrl);

    $path = $urlComponents['path'] ?? "";

    if ($this->urlParams == FALSE) {

      $this->pageUrl = $urlComponents['scheme'] . "://" .
            $urlComponents['host'] . $path;
    }
    else {

      $query = $urlComponents['query'] ?? "";
      $querySorted = "";

      if ($query != "") {
        $queryComponents = explode("&", $query);
        sort($queryComponents);
        $querySorted = "?" . implode("&", $queryComponents);
      }

      $this->pageUrl = $urlComponents['scheme'] . "://" .
            $urlComponents['host'] . $path . $querySorted;

    }
  }

  /**
   * Get Schema Markup for the given page.
   *
   * First checks the drupal cache and returns an item there
   * Second if not found in the cache search the CDN return that
   * If nothing is found false is returnd.
   *
   * @return mixed
   *   String if markup is found, false if no markup is found.
   *
   * @throws Exception
   *    On Guzzle Failure or Network failure.
   */
  public function get() {
    $existingItem = $this->checkCache();
    if ($existingItem) {
      if (property_exists($existingItem, "data")) {
        return $existingItem->data;
      }
    }

    $schemadata = $this->cdnGet();

    if ($schemadata) {
      $this->setCache($schemadata);
    }

    return $schemadata;
  }

  /**
   * CDN Get.
   *
   * Fetches directly from the Schema App CDN and does not cache.
   *
   * @return mixed
   *   String or FALSE if no schema exists for this URL.
   */
  public function cdnGet() {
    $userGraph = str_replace('http://schemaapp.com/db/',
        "",
        $this->accountId
      );

    $encodedUrl = rtrim(base64_encode($this->pageUrl), "=");

    $api = self::APIBASE . $userGraph . "/$encodedUrl";

    $guzzle = $this->getGuzzle();

    $cdnResponse = $guzzle->request(
      'GET', $api, [
        'headers' => [
          'X-API-KEY' => $this->apiKey,
        ],
        'connect_timeout' => $this->requestTimeout,
      ]
    );

    $statusCode = $cdnResponse->getStatusCode();
    if ($statusCode < 200 || $statusCode > 300) {
      return FALSE;
    }

    $schemadata = $cdnResponse->getBody()->getContents();

    return $schemadata === "" ? FALSE : $schemadata;

  }

  /**
   * Check Cache.
   *
   * Checks the cache for markup for the current page.
   *
   * @return mixed
   *   String of schema markup on success | False if it couldn't be found.
   */
  public function checkCache() {
    $hashedUrl = \md5($this->pageUrl);
    $item = \Drupal::cache()->get(self::CACHE_ID . $hashedUrl);

    return $item;
  }

  /**
   * Set Cache.
   *
   * @param string $contents
   *   The contents to set in the cache.
   * @param int $duration
   *   The period of time in seconds to cache contents.
   */
  public function setCache(string $contents, $duration = NULL) {
    $hashedUrl = \md5($this->pageUrl);
    $itemKey = self::CACHE_ID . $hashedUrl;

    if ($duration === NULL) {
      $duration = $this->cacheLifeTime;
    }

    \Drupal::cache()->set($itemKey, $contents, $duration);
  }

  /**
   * Return Guzzle Client from Drupal.
   *
   * @return \GuzzleHttp\Client
   *   Drupal default guzzle client.
   */
  protected function getGuzzle() {
    return \Drupal::httpClient();
  }

}
